#include "AT45DB321D.h"
#include "dac.h"
#include "microdelay.h"
#include "audio.h"
#include <tempsensor.h>
#include <stdint.h>

/* From 1343-demo-setup.c */
extern temperature_t 	temp;
extern int8_t 		critical_temp;
extern void update_temp(void);

/*****************************************************************************
 ** Function name:		ReadManufacturerID
 **
 ** Descriptions:		Reading Manufacturer and Device ID from Flash to 
 **                             the buffer,the parameter is buffer to coy the datas.
 **
 ** parameters:			Buffer pointer 
 **
 ** Returned value:		None.
 ** 
 *****************************************************************************/ 
uint8_t read_audio_manufacturer_id(void) 
{
	uint8_t buf[2];

	/* Set chip select low */
	gpio_set_pin(FCS, 0);	
	spi_tx_rx(DF_READ_MANUFACTURER_AND_DEVICE_ID);
	buf[0] = spi_tx_rx(0xff);
	buf[1] = spi_tx_rx(0xff);
	
	/* release SPI bus for use by other sensors */ 
	gpio_set_pin(FCS, 1);
	return buf[0];
} 

/*****************************************************************************
 ** Function name:		flashmem_init
 **
 ** Descriptions:		Enable Flash Chipselect and Reset pins
 **
 ** parameters:			None
 **
 ** Returned value:		None
 **
 *****************************************************************************/
void flashmem_init(void)
{
	/* Initialize GPIO controlle */
	gpio_init();
	
	/* Enable Chipselect and FlashReset pins */
	gpio_enable_pin(FCS);
	gpio_enable_pin(FRST);
	
	/* Set Ouput Chipselect and FlashReset pins */
	gpio_direction_output(FCS, 1);
	gpio_direction_output(FRST, 1);
	
	/* set FlashReset pin into high */
	gpio_set_pin(FRST, 1); // equal to - gpio_direction_output(FRST, 1)
}

/*****************************************************************************
 ** Function name:		ByteRead
 **
 ** Descriptions:		Read a Byte from flash, the first
 **				parameter is the Page Address, the 2nd parameter is
 **				byte address within the page. 
 **
 ** parameters:			Page Address, byte address
 **
 ** Returned value:		byte data
 **
*****************************************************************************/ 
uint8_t read_byte(uint32_t PageAdr, uint16_t ByteAdr) 
{
	uint8_t rdateseq[8];
	uint8_t temp;
	
	/* Send page read command, page address, byte address 
	   one by one MSB First */ 
	rdateseq[0] = DF_PAGE_READ;
	rdateseq[1] = (uint8_t)(PageAdr >> 6);
	rdateseq[2] = (uint8_t)(PageAdr << 2 | ByteAdr >> 8);
	rdateseq[3] = (uint8_t) ByteAdr;
	
	/*Four dont care bytes that follow the address bytes are needed to 
	  initialize the read operation. */ 
	rdateseq[4] = 0x00;
	rdateseq[5] = 0x00;
	rdateseq[6] = 0x00;
	rdateseq[7] = 0x00;

        /* Set chip select low */
	gpio_set_pin(FCS, 0);	

        /*write page and byte address to read a data byte */
	spi_write(rdateseq, 8);	

        /* reading data from the given address */
	temp = spi_tx_rx(0xFF);	
	
	/* Set chip select high */
	gpio_set_pin(FCS, 1);	
	return temp;
}

void play(void)
{
	srate    = 8;
	paddress = 0; 
	eadd     = 4622; 
	baddress = 0;

	while (true) {
		update_temp();
		if (temp.degree < (critical_temp - 2)) {
			break;
		}

                if ((paddress >  eadd) && (baddress == 0)) {
			paddress == 0;
			break;
                } else {
			dac_write(read_byte(paddress,baddress));
			baddress++;
			if (baddress == PGSIZE) {
				baddress = 0;
				paddress++;
			}
			microdelay(srate);
			event_poll_once();
		}
	}
}
